@@ -167,6 +167,8 @@ struct proc_dir_entry *proc_fs_cifs;
 read_proc_t cifs_txanchor_read;
 static read_proc_t cifsFYI_read;
 static write_proc_t cifsFYI_write;
+static read_proc_t oplockEnabled_read;
+static write_proc_t oplockEnabled_write;
 static read_proc_t traceSMB_read;
 static write_proc_t traceSMB_write;
 static read_proc_t multiuser_mount_read;
@@ -190,9 +192,8 @@ void cifs_proc_init(void)
 	create_proc_read_entry("TotalOps", 0, proc_fs_cifs,
 			       cifs_total_xid_read, 0);
 
-	pde =
-	    create_proc_read_entry("cifsFYI", 0, proc_fs_cifs,
-				   cifsFYI_read, 0);
+	pde = create_proc_read_entry("cifsFYI", 0, proc_fs_cifs,
+				     cifsFYI_read, 0);
 	if (pde)
 		pde->write_proc = cifsFYI_write;
 
@@ -202,6 +203,12 @@ void cifs_proc_init(void)
 	if (pde)
 		pde->write_proc = traceSMB_write;
 
+	pde = create_proc_read_entry("oplockEnabled", 0, proc_fs_cifs,
+				     oplockEnabled_read, 0);
+	if (pde)
+		pde->write_proc = oplockEnabled_write;
+
+
 	pde =
 	    create_proc_read_entry("MultiuserMount", 0, proc_fs_cifs,
 				   multiuser_mount_read, 0);
@@ -220,6 +227,7 @@ void cifs_proc_clean(void)
 	remove_proc_entry("MaxSimultaneousOps", proc_fs_cifs);
 	remove_proc_entry("TotalOps", proc_fs_cifs);
 	remove_proc_entry("MultiuserMount", proc_fs_cifs);
+	remove_proc_entry("oplockEnabled", proc_fs_cifs);
 	remove_proc_entry("cifs", proc_root_fs);
 }
 
@@ -260,6 +268,44 @@ static int cifsFYI_write(struct file *file, const char *buffer,
 
 	return count;
 }
+
+static int oplockEnabled_read(char *page, char **start, off_t off,
+			      int count, int *eof, void *data)
+{
+	int len;
+
+	len = sprintf(page, "%d\n", oplockEnabled);
+
+	len -= off;
+	*start = page + off;
+
+	if (len > count)
+		len = count;
+	else
+		*eof = 1;
+
+	if (len < 0)
+		len = 0;
+
+	return len;
+}
+static int oplockEnabled_write(struct file *file, const char *buffer,
+			       unsigned long count, void *data)
+{
+	char c;
+	int rc;
+
+	rc = get_user(c, buffer);
+	if (rc)
+		return rc;
+	if (c == '0' || c == 'n' || c == 'N')
+		oplockEnabled = 0;
+	else if (c == '1' || c == 'y' || c == 'Y')
+		oplockEnabled = 1;
+
+	return count;
+}
+
 static int traceSMB_read(char *page, char **start, off_t off, int count,
 			 int *eof, void *data)
 {
