@@ -86,20 +86,12 @@ enum failureFlagval {
 
 /*
  *****************************************************************
- * All typedefs and structs go here
+ * All typedefs (except the CIFS PDUs themselves) and
+ * the globally interesting structs should go here
  *****************************************************************
  */
 
-
-struct DirChild {
-	struct DirChild *next1;
-	struct vnode *vp;
-	int fid;
-};
-
-
 struct TCP_Server_Info {
-	struct DirChild dirInfo;
 	char server_Name[SERVER_NAME_LEN_WITH_NULL];	/* 15 chars + X'20'in 16th */
 	char unicode_server_Name[SERVER_NAME_LEN_WITH_NULL * 2];	/* Unicode version of server_Name */
 	struct socket *ssocket;
@@ -195,7 +187,7 @@ struct smbTconInfo {
 };
 
 /*
- * This info hangs off the smbFileInfo structure.  This is used to track
+ * This info hangs off the cifsFileInfo structure.  This is used to track
  * byte stream locks on the file
  */
 struct smbLockInfo {
@@ -207,23 +199,32 @@ struct smbLockInfo {
 /*
  * One of these for each open instance of a file
  */
-struct smbFileInfo {
-	struct smbTconInfo *tcon;	/* pointer to owning TCON */
-	struct smbFileInfo *tlist;	/* pointer to next fid owned by tcon *//* BB replace with list, atomicize */
-	struct smbFileInfo *nfip;	/* next fip associated with file *//* BB replace with list and atomicize */
-	struct smbLockInfo *lockList;	/* BB replace with list and atomicize */
-	int mode;		/* Mode with which file was opened */
+struct cifsFileInfo {
+	struct cifsFileInfo *tlist;	/* pointer to next fid owned by tcon *//* BB replace with list, atomicize */
+	struct cifsFileInfo *nfip;	/* next fip associated with file *//* BB replace with list and atomicize */
 	unsigned int uid;	/* allows you to find which FileInfo structure */
-	/* to use for i/o  filled in by msgOpen */
-	unsigned int pid;	/* process id who opened file */
+	pid_t pid;		/* process id who opened file */
 	__u16 netfid;		/* file id from remote */
 	/* BB add lock scope info here if needed */ ;
 	/* lock scope id (0 if none) */
-	atomic_t inUse;		/* concurrent users of this open instance */
 	int endOfSearch:1;	/* we have reached end of search */
 	int closePend:1;	/* file is marked to close */
 };
 
+/*
+ * One of these for each file inode
+ */
+
+struct cifsInodeInfo {
+	struct smbLockInfo *lockList;	/* BB replace with list and atomicize */
+	__u32 cifsAttrs;	/* e.g. DOS archive bit, sparse, compressed, system etc. */
+	atomic_t inUse;		/* num concurrent users (local openers cifs) of file */
+	unsigned long time;	/* jiffies of last update/check of inode */
+	int clientCanCache:1;	/* oplocked.  We need to extend cases beyond this i.e. what
+				   if file read-only or if file locked? or if file on r/o vol? */
+};
+
+/* one of these for every pending CIFS request to the server */
 struct mid_q_entry {
 	struct list_head qhead;	/* mids waiting on reply from this server */
 	__u16 mid;		/* multiplex id */
@@ -309,3 +310,4 @@ GLOBAL_EXTERN atomic_t midCount;
 /* Misc globals */
 GLOBAL_EXTERN unsigned int multiuser_mount;	/* if enabled allows new sessions to be established on existing mount if we */
 					    /* have the uid/password or Kerberos credential or equivalent for current user */
+GLOBAL_EXTERN unsigned int oplockEnabled;
