@@ -357,7 +357,7 @@ int CIFSSMBMkDir(const int xid, const struct smbTconInfo *tcon,
 int CIFSSMBOpen(const int xid, const struct smbTconInfo *tcon,
 		const char *fileName, const int openDisposition,
 		const int access_flags, const int omode, __u16 * netfid,
-		const struct nls_table *nls_codepage)
+		int *pOplock, const struct nls_table *nls_codepage)
 {
 	int rc = -EACCES;
 
@@ -382,27 +382,30 @@ int CIFSSMBOpen(const int xid, const struct smbTconInfo *tcon,
 				  , nls_codepage);
 		name_len++;	/* trailing null */
 		name_len *= 2;
-		pSMB->NameLength = name_len;
+		pSMB->NameLength = cpu_to_le16(name_len);
 	} else {		/* BB improve the check for buffer overruns BB */
 
 		pSMB->ByteCount = 0;	/* no pad */
 		name_len = strnlen(fileName, 530);
 		name_len++;	/* trailing null */
-		pSMB->NameLength = name_len;
+		pSMB->NameLength = cpu_to_le16(name_len);
 		strncpy(pSMB->fileName, fileName, name_len);
 	}
-
-	pSMB->DesiredAccess = access_flags;
+	if (*pOplock)
+		pSMB->OpenFlags = cpu_to_le32(REQ_OPLOCK);
+	pSMB->DesiredAccess = cpu_to_le32(access_flags);
 	pSMB->AllocationSize = 0;
 	pSMB->FileAttributes = ATTR_NORMAL;	/* XP does not handle ATTR_POSIX_SEMANTICS */
 	if ((omode & S_IWUGO) == 0)
 		pSMB->FileAttributes |= ATTR_READONLY;	/* BB might as well do this until CIFS ACL support is put in */
-	pSMB->ShareAccess = FILE_SHARE_ALL;
-	pSMB->CreateDisposition = openDisposition;
-	pSMB->CreateOptions = CREATE_NOT_DIR;	/* BB what are these? BB */
-	pSMB->ImpersonationLevel = SECURITY_IMPERSONATION;	/* BB ?? BB */
+	pSMB->FileAttributes = cpu_to_le32(pSMB->FileAttributes);
+	pSMB->ShareAccess = cpu_to_le32(FILE_SHARE_ALL);
+	pSMB->CreateDisposition = cpu_to_le32(openDisposition);
+	pSMB->CreateOptions = cpu_to_le32(CREATE_NOT_DIR);	/* BB what are these? BB */
+	pSMB->ImpersonationLevel = cpu_to_le32(SECURITY_IMPERSONATION);	/* BB ?? BB */
 	pSMB->SecurityFlags =
-	    SECURITY_CONTEXT_TRACKING | SECURITY_EFFECTIVE_ONLY;
+	    cpu_to_le32(SECURITY_CONTEXT_TRACKING |
+			SECURITY_EFFECTIVE_ONLY);
 
 	pSMB->ByteCount += name_len;
 	pSMB->hdr.smb_buf_length += pSMB->ByteCount;
@@ -412,9 +415,11 @@ int CIFSSMBOpen(const int xid, const struct smbTconInfo *tcon,
 			 (struct smb_hdr *) pSMBr, &bytes_returned, 0);
 	if (rc) {
 		cERROR(1, ("\nSend error in Open = %d\n", rc));
-	} else
-		*netfid = pSMBr->Fid;
-
+	} else {
+		*pOplock = pSMBr->OplockLevel;	/* one byte no need to le_to_cpu */
+		*netfid = pSMBr->Fid;	/* cifs fid stays in le */
+		/* Do we care about the CreateAction in any cases? */
+	}
 	if (pSMB)
 		buf_release(pSMB);
 	if (pSMBr)
@@ -1315,7 +1320,7 @@ int CIFSFindFirst(const int xid, const struct smbTconInfo *tcon,
 		cFYI(1, ("\nError in FindFirst = %d\n", rc));
 	} else {		/* decode response */
 		/* BB add safety checks for these memcpys */
-		if (pSMBr->hdr.Flags2 & le16_to_cpu(SMBFLG2_UNICODE))
+		if (pSMBr->hdr.Flags2 & SMBFLG2_UNICODE)
 			*pUnicodeFlag = TRUE;
 		else
 			*pUnicodeFlag = FALSE;
@@ -1373,11 +1378,9 @@ int CIFSFindNext(const int xid, const struct smbTconInfo *tcon,
 	pSMB->SubCommand = TRANS2_FIND_NEXT;
 	pSMB->ByteCount = pSMB->TotalParameterCount + 1 /* pad */ ;
 	pSMB->SearchHandle = searchHandle;
-	pSMB->SearchCount =
-	    min_t(int, CIFS_MAX_MSGSIZE / sizeof(FILE_DIRECTORY_INFO),
-		  findParms->SearchCount);
-	findParms->SearchCount = 0;	/* reset to zero in case of error */
-	/* test for Unix extensions */
+	findParms->SearchCount = 0;	/* set to zero in case of error */
+	pSMB->SearchCount = cpu_to_le16(CIFS_MAX_MSGSIZE / sizeof(FILE_DIRECTORY_INFO));  
+    /* test for Unix extensions */
 	if (tcon->ses->capabilities & CAP_UNIX) {
 		pSMB->InformationLevel = cpu_to_le16(SMB_FIND_FILE_UNIX);
 		*pUnixFlag = TRUE;
@@ -1491,7 +1494,7 @@ int CIFSGetDFSRefer(const int xid, const struct smbTconInfo *tcon,
 		buf_release(pSMB);
 	if (pSMBr)
 		buf_release(pSMBr);
-        return rc;
+	return rc;
 }
 
 int CIFSSMBQFSInfo(const int xid, const struct smbTconInfo *tcon,
