@@ -36,7 +36,7 @@ void renew_parental_timestamps(struct dentry *direntry)
 	do {
 		direntry->d_time = jiffies;
 		direntry = direntry->d_parent;
-	} while (!IS_ROOT(direntry));
+	} while (!IS_ROOT(direntry));	/* BB for DFS case should stop at the root of share which could be lower than root of this mount due to implicit dfs connections */
 }
 
 /* Note: caller must free return buffer */
@@ -48,9 +48,11 @@ char *build_path_from_dentry(struct dentry *direntry)
 
 	for (temp = direntry; !IS_ROOT(temp);) {
 		namelen += (1 + temp->d_name.len);
+        cFYI(1,(" len %d ",namelen));
 		temp = temp->d_parent;
 	}
 	namelen += 1;		/* allow for trailing null */
+    cFYI(1,("Final namelength (in build_path): %d ",namelen)); /* BB remove */
 	full_path = kmalloc(namelen, GFP_KERNEL);
 	namelen--;
 	full_path[namelen] = 0;	/* trailing null */
@@ -62,6 +64,7 @@ char *build_path_from_dentry(struct dentry *direntry)
 			full_path[namelen] = '\\';
 			strncpy(full_path + namelen + 1, temp->d_name.name,
 				temp->d_name.len);
+            cFYI(1,(" name: %s ",full_path + namelen)); /* BB remove */
 		}
 		temp = temp->d_parent;
 	}
@@ -117,6 +120,7 @@ int cifs_create(struct inode *inode, struct dentry *direntry, int mode)
 {
 	int rc = -ENOENT;
 	int xid;
+	int oplock = FALSE;	/* no need to oplock when we are not going to read from the file */
 	__u16 fileHandle;
 	struct cifs_sb_info *cifs_sb;
 	struct smbTconInfo *pTcon;
@@ -135,7 +139,7 @@ int cifs_create(struct inode *inode, struct dentry *direntry, int mode)
 	rc = CIFSSMBOpen(xid, pTcon, full_path, FILE_OVERWRITE_IF,
 			 GENERIC_ALL
 			 /* 0x20197 was used previously */ , mode,
-			 &fileHandle, cifs_sb->local_nls);
+			 &fileHandle, &oplock, cifs_sb->local_nls);
 	if (rc) {
 		cFYI(1, ("\ncifs_create returned 0x%x ", rc));
 	} else {
@@ -257,8 +261,8 @@ static int cifs_d_revalidate(struct dentry *direntry, int flags)
 
 	if (direntry->d_inode) {
 		cFYI(1,
-		     ("In cifs_d_revalidate, name = %s and inode = 0x%p and dentry 0x%p\n",
-		      direntry->d_name.name, direntry->d_inode, direntry));
+		     ("In cifs_d_revalidate, name = %s and inode = 0x%p with count %d with time %ld and dentry 0x%p with time %ld\n",
+		      direntry->d_name.name, direntry->d_inode, direntry->d_inode->i_count, direntry->d_inode->i_atime, direntry, direntry->d_time));
 	} else {
 		cFYI(1,
 		     ("In cifs_d_revalidate with no inode but name = %s and dentry 0x%p\n",
@@ -267,14 +271,14 @@ static int cifs_d_revalidate(struct dentry *direntry, int flags)
 	return isValid;
 }
 
-static int cifs_d_delete(struct dentry *direntry)
+/* static int cifs_d_delete(struct dentry *direntry)
 {
 	int rc = 0;
 
 	cFYI(1, ("In cifs d_delete, name = %s\n", direntry->d_name.name));
 
 	return rc;
-}
+}     */
 
 static struct dentry_operations cifs_dentry_ops = {
 	d_revalidate:cifs_d_revalidate,
