@@ -35,7 +35,7 @@
 int cifs_open(struct inode *inode, struct file *file)
 {
 	int rc = -EACCES;
-	int xid;
+	int xid, oplock;
 	struct cifs_sb_info *cifs_sb;
 	struct smbTconInfo *pTcon;
 	char *full_path = NULL;
@@ -96,20 +96,28 @@ int cifs_open(struct inode *inode, struct file *file)
 	if (file->f_flags & O_CREAT)
 		disposition = FILE_OPEN_IF;	/* open if exists, create if does not exist */
 
+	/* BB finish adding in oplock support BB */
+	if (oplockEnabled)
+		oplock = TRUE;
+	else
+		oplock = FALSE;
+
+
 	/* BB pass O_SYNC flag through on file attributes .. BB */
 	rc = CIFSSMBOpen(xid, pTcon, full_path, disposition, desiredAccess,
 			 0x3FF
 			 /* i.e. 777 dummy value, ignored for time being */
-			 , &netfid, cifs_sb->local_nls);
+			 , &netfid, &oplock, cifs_sb->local_nls);
 	if (rc) {
 		cFYI(1, ("\ncifs_open returned 0x%x ", rc));
+		cFYI(1, (" oplock: %d ", oplock));
 	} else {
 		file->private_data =
-		    kmalloc(sizeof(struct smbFileInfo), GFP_KERNEL);
+		    kmalloc(sizeof(struct cifsFileInfo), GFP_KERNEL);
 		if (file->private_data) {
 			memset(file->private_data, 0,
-			       sizeof(struct smbFileInfo));
-			((struct smbFileInfo *) file->private_data)->
+			       sizeof(struct cifsFileInfo));
+			((struct cifsFileInfo *) file->private_data)->
 			    netfid = netfid;
 		}
 	}
@@ -126,8 +134,8 @@ int cifs_close(struct inode *inode, struct file *file)
 	int xid;
 	struct cifs_sb_info *cifs_sb;
 	struct smbTconInfo *pTcon;
-	struct smbFileInfo *pSMBFile =
-	    (struct smbFileInfo *) file->private_data;
+	struct cifsFileInfo *pSMBFile =
+	    (struct cifsFileInfo *) file->private_data;
 
 	cFYI(1, ("\n  inode = 0x%p with ", inode));
 
@@ -152,20 +160,20 @@ int cifs_closedir(struct inode *inode, struct file *file)
 {
 	int rc = 0;
 	int xid;
-	struct smbFileInfo *pSMBFileStruct =
-	    (struct smbFileInfo *) file->private_data;
+	struct cifsFileInfo *pSMBFileStruct =
+	    (struct cifsFileInfo *) file->private_data;
 
-	cFYI(1, ("\n  inode = 0x%p with ", inode));
+	cFYI(1, ("\nClosedir inode = 0x%p with ", inode));
 
 	xid = GetXid();
 
 	if (pSMBFileStruct) {
+        cFYI(1,("\nFreeing private data in close dir"));
 		kfree(file->private_data);
 		file->private_data = NULL;
 	}
 	FreeXid(xid);
 	return rc;
-
 }
 
 
@@ -233,14 +241,14 @@ int cifs_lock(struct file *file, int cmd, struct file_lock *pfLock)
 
 	if (IS_GETLK(cmd)) {
 		rc = CIFSSMBLock(xid, pTcon,
-				 ((struct smbFileInfo *) file->
+				 ((struct cifsFileInfo *) file->
 				  private_data)->netfid,
 				 length,
 				 pfLock->fl_start, 0, 1, lockType,
 				 0 /* wait flag */ );
 		if (rc == 0) {
 			rc = CIFSSMBLock(xid, pTcon,
-					 ((struct smbFileInfo *) file->
+					 ((struct cifsFileInfo *) file->
 					  private_data)->netfid,
 					 length,
 					 pfLock->fl_start,
@@ -264,7 +272,7 @@ int cifs_lock(struct file *file, int cmd, struct file_lock *pfLock)
 	}
 
 	rc = CIFSSMBLock(xid, pTcon,
-			 ((struct smbFileInfo *) file->private_data)->
+			 ((struct cifsFileInfo *) file->private_data)->
 			 netfid, length,
 			 pfLock->fl_start, numUnlock, numLock, lockType,
 			 0 /* wait flag */ );
@@ -304,7 +312,7 @@ ssize_t cifs_write(struct file * file, const char *write_data,
 	for (total_written = 0; write_size > total_written;
 	     total_written += bytes_written) {
 		rc = CIFSSMBWrite(xid, pTcon,
-				  ((struct smbFileInfo *) file->
+				  ((struct cifsFileInfo *) file->
 				   private_data)->netfid,
 				  write_size - total_written, *poffset,
 				  &bytes_written,
@@ -361,7 +369,7 @@ ssize_t cifs_read(struct file * file, char *read_data, size_t read_size,
 	for (total_read = 0; read_size > total_read;
 	     total_read += bytes_read) {
 		rc = CIFSSMBRead(xid, pTcon,
-				 ((struct smbFileInfo *) file->
+				 ((struct cifsFileInfo *) file->
 				  private_data)->netfid,
 				 read_size - total_read, *poffset,
 				 &bytes_read, read_data + total_read);
@@ -383,11 +391,19 @@ ssize_t cifs_read(struct file * file, char *read_data, size_t read_size,
 void fill_in_inode(struct inode *tmp_inode,
 		   FILE_DIRECTORY_INFO * pfindData, int *pobject_type)
 {
+	struct cifsInodeInfo *cifsInfo =
+	    (struct cifsInodeInfo *) &(tmp_inode->u);
+
+	cifsInfo->cifsAttrs = pfindData->ExtFileAttributes;
+	cifsInfo->time = jiffies;
+	atomic_inc(&cifsInfo->inUse);  /* inc on every refresh of inode info */
+
 	/* Linux can not store file creation time unfortunately so we ignore it */
 	tmp_inode->i_atime = cifs_NTtimeToUnix(pfindData->LastAccessTime);
 	tmp_inode->i_mtime = cifs_NTtimeToUnix(pfindData->LastWriteTime);
 	tmp_inode->i_ctime = cifs_NTtimeToUnix(pfindData->ChangeTime);
-	tmp_inode->i_mode = S_IRWXUGO;	/* 777 perms */
+    /* should we treat the dos attribute of read-only as read-only mode bit e.g. 555 */
+	tmp_inode->i_mode = S_IALLUGO & ~(S_ISUID | S_IXGRP); /* 2767 perms - indicate mandatory locking */
 	cFYI(0,
 	     ("\nCIFS FFIRST: Attributes came in as 0x%x",
 	      pfindData->ExtFileAttributes));
@@ -396,21 +412,22 @@ void fill_in_inode(struct inode *tmp_inode,
 		tmp_inode->i_mode |= S_IFLNK;	/* BB can this and S_IFREG or S_IFDIR be set at same time as in Windows? */
 	} else if (pfindData->ExtFileAttributes & ATTR_DIRECTORY) {
 		*pobject_type = DT_DIR;
+        tmp_inode->i_mode = S_IRWXUGO; /* override default perms since we do not lock dirs */
 		tmp_inode->i_mode |= S_IFDIR;
 	} else {
 		*pobject_type = DT_REG;
 		tmp_inode->i_mode |= S_IFREG;
 	}			/* BB add code here - to validate if device or weird share type or odd device type? BB */
 
-
+    /* can not fill in nlink here as we can in both qpathinfo versions and in unix search */
 	tmp_inode->i_size = pfindData->EndOfFile;
 	tmp_inode->i_blocks =
 	    do_div(pfindData->AllocationSize, tmp_inode->i_blksize);
-	cFYI(1,
+	cFYI(0,
 	     ("\nFinddata alloc size (from smb) %lld",
 	      pfindData->AllocationSize));
 	if (pfindData->AllocationSize < pfindData->EndOfFile)
-		cFYI(1, ("\nServer inconsistency Error: it says allocation size less than end of file "));	/* BB remove */
+		cFYI(0, ("\nServer inconsistency Error: it says allocation size less than end of file "));	/* BB remove */
 	cFYI(1,
 	     ("\nCIFS FFIRST: Size %ld and blocks %ld ",
 	      (unsigned long) tmp_inode->i_size, tmp_inode->i_blocks));
@@ -435,6 +452,11 @@ void fill_in_inode(struct inode *tmp_inode,
 void unix_fill_in_inode(struct inode *tmp_inode,
 			FILE_UNIX_INFO * pfindData, int *pobject_type)
 {
+	struct cifsInodeInfo *cifsInfo =
+	    (struct cifsInodeInfo *) &(tmp_inode->u);
+	cifsInfo->time = jiffies;
+	atomic_inc(&cifsInfo->inUse);
+
 	tmp_inode->i_atime = cifs_NTtimeToUnix(pfindData->LastAccessTime);
 	tmp_inode->i_mtime =
 	    cifs_NTtimeToUnix(pfindData->LastModificationTime);
@@ -476,9 +498,9 @@ void unix_fill_in_inode(struct inode *tmp_inode,
 	tmp_inode->i_size = pfindData->EndOfFile;
 	tmp_inode->i_blocks =
 	    do_div(pfindData->NumOfBytes, tmp_inode->i_blksize);
-	cFYI(1, ("\nFinddata alloc size (from smb) %lld", pfindData->NumOfBytes));	/* BB remove */
+	cFYI(0, ("\nFinddata alloc size (from smb) %lld", pfindData->NumOfBytes));	/* BB remove */
 	if (pfindData->NumOfBytes < pfindData->EndOfFile)
-		cFYI(1, ("\nServer inconsistency Error: it says allocation size less than end of file "));	/* BB remove */
+		cFYI(0, ("\nServer inconsistency Error: it says allocation size less than end of file "));	/* BB remove */
 	cFYI(1, ("\nCIFS FFIRST: Size %ld and blocks %ld ", (unsigned long) tmp_inode->i_size, tmp_inode->i_blocks));	/* BB remove */
 	if (S_ISREG(tmp_inode->i_mode)) {
 		cFYI(1, (" File inode "));
@@ -499,94 +521,152 @@ void unix_fill_in_inode(struct inode *tmp_inode,
 	}
 }
 
+void construct_dentry(struct qstr * qstring,struct file * file, struct inode ** ptmp_inode,struct dentry ** pnew_dentry)
+{
+        struct dentry *tmp_dentry;  
+        struct cifs_sb_info *cifs_sb;
+        struct smbTconInfo *pTcon;
+
+        cFYI(1,("\nFor %s ",qstring->name));
+        cifs_sb = (struct cifs_sb_info *) (&(file->f_dentry->d_sb->u));
+        pTcon = cifs_sb->tcon;
+
+        qstring->hash = full_name_hash(qstring->name, qstring->len);
+        tmp_dentry = d_lookup(file->f_dentry, qstring);
+        if (tmp_dentry) {
+	            cFYI(1, (" existing dentry with inode 0x%p ", tmp_dentry->d_inode));	/* BB remove */
+	            *ptmp_inode = tmp_dentry->d_inode;
+	    /* BB overwrite the old name? i.e. tmp_dentry->d_name and tmp_dentry->d_name.len ?? */
+        } else {
+	            tmp_dentry = d_alloc(file->f_dentry, qstring);
+	            *ptmp_inode = new_inode(file->f_dentry->d_sb);
+                cFYI(1,("\nAlloc new inode %p ",*ptmp_inode));
+	            tmp_dentry->d_op = &cifs_dentry_ops;
+	            cFYI(1, (" instantiate dentry 0x%p with inode 0x%p ",
+	            tmp_dentry, *ptmp_inode));
+	            d_instantiate(tmp_dentry, *ptmp_inode);
+                d_rehash(tmp_dentry);
+        }
+
+        tmp_dentry->d_time = jiffies;
+        (*ptmp_inode)->i_blksize =
+                (pTcon->ses->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00; 
+        *pnew_dentry = tmp_dentry;
+}
+
+void cifs_filldir(struct qstr * pqstring, FILE_DIRECTORY_INFO * pfindData, struct file * file, filldir_t filldir, void * direntry)
+{
+    struct inode * tmp_inode;
+    struct dentry * tmp_dentry;
+    int object_type;
+
+    pqstring->name = pfindData->FileName;
+    pqstring->len = pfindData->FileNameLength;
+
+    construct_dentry(pqstring, file,&tmp_inode,&tmp_dentry);
+
+    fill_in_inode(tmp_inode, pfindData, &object_type);
+    filldir(direntry,pfindData->FileName,pqstring->len, file->f_pos, tmp_inode->i_ino, object_type); 
+    dput(tmp_dentry);
+}
+
+void cifs_filldir_unix(struct qstr * pqstring, FILE_UNIX_INFO * pUnixFindData, struct file * file, filldir_t filldir, void * direntry)
+{
+    struct inode * tmp_inode;
+    struct dentry * tmp_dentry;
+    int object_type;
+
+    pqstring->name = pUnixFindData->FileName;
+    pqstring->len =
+        strnlen(pUnixFindData->FileName, MAX_PATHCONF);
+
+    construct_dentry(pqstring, file,&tmp_inode,&tmp_dentry);
+
+	unix_fill_in_inode(tmp_inode, pUnixFindData, &object_type);
+    filldir(direntry,pUnixFindData->FileName,pqstring->len, file->f_pos, tmp_inode->i_ino, object_type); 
+    dput(tmp_dentry);
+}
+
+
+
+
 int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 {
 	int rc = 0;
-	int rc2 = 0;
-	int total_returned_this_readdir = 0;
-	int last_index = 0;
 	int xid, i;
-	int existing;
-	int done_search = FALSE;
 	int Unicode = FALSE;
 	int UnixSearch = FALSE;
 	__u16 searchHandle;
 	struct cifs_sb_info *cifs_sb;
-	int object_type;
 	struct smbTconInfo *pTcon;
+    struct cifsFileInfo * cifsFile = NULL;
 	char *full_path = NULL;
 	char *data;
-	struct inode *tmp_inode;
-	struct dentry *tmp_dentry;
 	struct qstr qstring;
 	T2_FFIRST_RSP_PARMS findParms;
 	T2_FNEXT_RSP_PARMS findNextParms;
 	FILE_DIRECTORY_INFO *pfindData;
-	FILE_UNIX_INFO *pUnixFindData;
+	FILE_UNIX_INFO *pfindDataUnix;
 
 	xid = GetXid();
 
 	data = kmalloc(4096, GFP_KERNEL);
 	pfindData = (FILE_DIRECTORY_INFO *) data;
-	pUnixFindData = (FILE_UNIX_INFO *) data;
 
 	cifs_sb = (struct cifs_sb_info *) (&(file->f_dentry->d_sb->u));
 	pTcon = cifs_sb->tcon;
-
+    
 	full_path = build_wildcard_path_from_dentry(file->f_dentry);
 
 	cFYI(1,
-	     ("\nFull path is %s with position %lld ", full_path,
-	      file->f_pos));
+	     ("\nFull path: %s start at: %lld ", full_path, file->f_pos));
+    
 
-	i = file->f_pos;
-	switch (i) {
+	switch ((int)file->f_pos) {            
 	case 0:
 		if (filldir
-		    (direntry, ".", 1, i, file->f_dentry->d_inode->i_ino,
+		    (direntry, ".", 1, file->f_pos, file->f_dentry->d_inode->i_ino,
 		     DT_DIR) < 0) {
-			cERROR(1,
-			       ("\nFilldir for current directory direntry failed "));
+			cERROR(1, ("\nFilldir for current dir failed "));
 			break;
-		}
-		i++;
+        }
 		file->f_pos++;
 		/* fallthrough */
 	case 1:
 		if (filldir
-		    (direntry, "..", 2, i,
+		    (direntry, "..", 2, file->f_pos,
 		     file->f_dentry->d_parent->d_inode->i_ino,
 		     DT_DIR) < 0) {
-			cERROR(1,
-			       ("\nFilldir for parent directory direntry failed "));
+			cERROR(1, ("\nFilldir for parent dir failed "));
 			break;
 		}
-		i++;
 		file->f_pos++;
 		/* fallthrough */
-
+    case 2:
+            /* Should we first check if file->private_data is null? */
 		rc = CIFSFindFirst(xid, pTcon, full_path, pfindData,
 				   &findParms, cifs_sb->local_nls,
 				   &Unicode, &UnixSearch);
-		cFYI(1, ("Count: %d  More avail entries: 0x%x ",
-			 findParms.SearchCount, findParms.EndofSearch));
+		cFYI(1,
+		     ("Count: %d  End: %d ", findParms.SearchCount,
+		      findParms.EndofSearch));
 
 		if (rc == 0) {
 			searchHandle = findParms.SearchHandle;
 			file->private_data =
-			    kmalloc(sizeof(struct smbFileInfo),
+			    kmalloc(sizeof(struct cifsFileInfo),
 				    GFP_KERNEL);
 			if (file->private_data) {
 				memset(file->private_data, 0,
-				       sizeof(struct smbFileInfo));
-				((struct smbFileInfo *) file->
-				 private_data)->netfid = searchHandle;
+				       sizeof(struct cifsFileInfo));
+                cifsFile = (struct cifsFileInfo *)file->private_data;
+                cifsFile->netfid = searchHandle;
 			}
+
+			renew_parental_timestamps(file->f_dentry);
+
 			for (i = 2; i < findParms.SearchCount + 2; i++) {
 				if (UnixSearch == FALSE) {
-					cFYI(1,
-					     ("\nFilename length: %d",
-					      pfindData->FileNameLength));
 					if (Unicode == TRUE)
 						pfindData->FileNameLength =
 						    cifs_strfromUCS_le
@@ -594,447 +674,133 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 						     (UniChar *)
 						     pfindData->FileName,
 						     (pfindData->
-						      FileNameLength)
-						     / 2,
+						      FileNameLength) / 2,
 						     cifs_sb->local_nls);
-					qstring.name = pfindData->FileName;
-					qstring.len =
-					    pfindData->FileNameLength;
-					qstring.hash =
-					    full_name_hash(qstring.name,
-							   qstring.len);
-					tmp_dentry =
-					    d_lookup(file->f_dentry,
-						     &qstring);
-					if (tmp_dentry) {
-						cFYI(1, ("\nExisting dentry found with inode 0x%p", tmp_dentry->d_inode));	/* BB remove */
-						existing = TRUE;
-						tmp_inode =
-						    tmp_dentry->d_inode;
-						/* BB overwrite the old name? i.e. tmp_dentry->d_name and tmp_dentry->d_name.len ?? */
-					} else {
-						existing = FALSE;
-						tmp_dentry =
-						    d_alloc(file->f_dentry,
-							    &qstring);
-						tmp_inode =
-						    new_inode(file->
-							      f_dentry->
-							      d_sb);
-						tmp_dentry->d_op =
-						    &cifs_dentry_ops;
-						cFYI(1,
-						     ("\nInstantiating dentry 0x%p with inode 0x%p ",
-						      tmp_dentry,
-						      tmp_inode));
-						d_instantiate(tmp_dentry,
-							      tmp_inode);
-						if (!existing)
-							d_rehash
-							    (tmp_dentry);
-					}
-					tmp_dentry->d_time = jiffies;
-					tmp_inode->i_blksize =
-					    (pTcon->ses->maxBuf -
-					     MAX_CIFS_HDR_SIZE) &
-					    0xFFFFFE00;
-					fill_in_inode(tmp_inode, pfindData,
-						      &object_type);
-					cFYI(1,
-					     ("\nFile name is %s ",
-					      pfindData->FileName));
-					if (((pfindData->FileNameLength !=
-					      1)
-					     || (pfindData->FileName[0] !=
-						 '.'))
-					    &&
-					    ((pfindData->FileNameLength !=
-					      2)
-					     || (pfindData->FileName[0] !=
-						 '.')
-					     || (pfindData->FileName[1] !=
-						 '.'))) {
-						filldir(direntry,
-							pfindData->
-							FileName,
-							pfindData->
-							FileNameLength, i,
-							tmp_inode->i_ino,
-							object_type);
-						file->f_pos++;
-					}
-					renew_parental_timestamps(file->
-								  f_dentry);
-					pfindData =
-					    (FILE_DIRECTORY_INFO
-					     *) ((char *)
-						 pfindData +
-						 pfindData->
-						 NextEntryOffset);
-					/* BB also check to make sure that it is not beyond the end of the SMB */
-				} else {	/* Unix Search */
+                    qstring.len = pfindData->FileNameLength;
+                    if (((qstring.len != 1)
+                         || (pfindData->FileName[0] != '.'))
+                        && ((qstring.len != 2)
+                        || (pfindData->FileName[0] != '.')
+                        || (pfindData->FileName[1] != '.'))) {
+                        cifs_filldir(&qstring, pfindData, file, filldir, direntry);
+                        file->f_pos++;
+                    }
+				} else {	/* UnixSearch */
+					pfindDataUnix =
+					    (FILE_UNIX_INFO *)pfindData;
 					if (Unicode == TRUE)
-						qstring.len =
-						    cifs_strfromUCS_le
-						    (pUnixFindData->
-						     FileName, (UniChar *)
-						     pUnixFindData->
+						qstring.len = cifs_strfromUCS_le
+						    (pfindDataUnix->
+						     FileName,
+						     (UniChar *)
+						     pfindDataUnix->
 						     FileName,
 						     MAX_PATHCONF,
 						     cifs_sb->local_nls);
-					else
-						qstring.len =
-						    strnlen(pUnixFindData->
-							    FileName,
-							    MAX_PATHCONF);
-					qstring.name =
-					    pUnixFindData->FileName;
-					qstring.hash =
-					    full_name_hash(qstring.name,
-							   qstring.len);
-					tmp_dentry =
-					    d_lookup(file->f_dentry,
-						     &qstring);
-					if (tmp_dentry) {
-						existing = TRUE;
-						cFYI(1, ("\nExisting dentry found for "));	/* BB remove */
-						tmp_inode =
-						    tmp_dentry->d_inode;
-					} else {
-						existing = FALSE;
-						tmp_dentry =
-						    d_alloc(file->f_dentry,
-							    &qstring);
-						tmp_inode =
-						    new_inode(file->
-							      f_dentry->
-							      d_sb);
-						tmp_dentry->d_op =
-						    &cifs_dentry_ops;
-						cFYI(1,
-						     ("\nInstantiating dentry 0x%p with inode 0x%p ",
-						      tmp_dentry,
-						      tmp_inode));
-						d_instantiate(tmp_dentry,
-							      tmp_inode);
-						if (!existing)
-							d_rehash
-							    (tmp_dentry);
-					}
-					tmp_dentry->d_time = jiffies;
-					tmp_inode->i_blksize =
-					    (pTcon->ses->maxBuf -
-					     MAX_CIFS_HDR_SIZE) &
-					    0xFFFFFE00;
-					unix_fill_in_inode(tmp_inode,
-							   pUnixFindData,
-							   &object_type);
-					cFYI(1,
-					     ("\nFile name is %s ",
-					      pUnixFindData->FileName));
-					if (((qstring.len != 1)
-					     || (pUnixFindData->
-						 FileName[0] != '.'))
-					    && ((qstring.len != 2)
-						|| (pUnixFindData->
-						    FileName[0] != '.')
-						|| (pUnixFindData->
-						    FileName[1] != '.'))) {
-						filldir(direntry,
-							pUnixFindData->
-							FileName,
-							qstring.len, i,
-							tmp_inode->i_ino,
-							object_type);
-						file->f_pos++;
-					}
-					renew_parental_timestamps(file->
-								  f_dentry);
-					pUnixFindData =
-					    (FILE_UNIX_INFO *) ((char *)
-								pUnixFindData
-								+
-								pUnixFindData->
-								NextEntryOffset);
-					/* BB also check to make sure that it is not beyond the end of the SMB */
-				}
+                    else
+                        qstring.len = strnlen(pfindDataUnix->FileName, MAX_PATHCONF);
+                    if (((qstring.len != 1)
+                         || (pfindDataUnix->FileName[0] != '.'))
+                        && ((qstring.len != 2)
+                        || (pfindDataUnix->FileName[0] != '.')
+                        || (pfindDataUnix->FileName[1] != '.'))) {
+                        cifs_filldir_unix(&qstring, pfindDataUnix, file, filldir, direntry);
+                        file->f_pos++;
+                    }
+				}              
+				pfindData = (FILE_DIRECTORY_INFO *) ((char *) pfindData + pfindData->NextEntryOffset);	/* works also for Unix find struct since this is the first field of both */
+				/* BB also should check to make sure that pointer is not beyond the end of the SMB */
 			}	/* end for loop */
-			last_index = i;
-			total_returned_this_readdir = last_index;
-			if (findParms.EndofSearch != 0)
-				done_search = TRUE;
+            if ((findParms.EndofSearch != 0) && cifsFile) {           
+               cifsFile->endOfSearch = TRUE;
+            }
 		} else {
-			done_search = TRUE;
-			rc = 0;	/* unless parent directory disappeared - do not return error here on 1st pass */
-		}		/* end if else */
-	default:
-		if (file->private_data)
-			searchHandle =
-			    ((struct smbFileInfo *) (file->private_data))->
-			    netfid;
-		else {
-			done_search = TRUE;
-			rc = -EBADF;
-			break;
+                if(cifsFile)
+                        cifsFile->endOfSearch = TRUE;
+                rc = 0;	/* unless parent directory disappeared - do not return error here (eg Access Denied or no more files) */            
 		}
-		while (done_search == FALSE) {
-			if (total_returned_this_readdir < 127) {
-				findNextParms.SearchCount =
-				    127 - (total_returned_this_readdir);
-			} else
-				break;
-			rc = CIFSFindNext(xid, pTcon, pfindData,
-					  &findNextParms, searchHandle, 0,
-					  &Unicode, &UnixSearch);
-			if ((rc == 0) && (findNextParms.SearchCount != 0)) {
-				for (i = 0; i < findNextParms.SearchCount;
-				     i++, total_returned_this_readdir++) {
-					if (UnixSearch == FALSE) {
-						if (Unicode == TRUE)
-							pfindData->
-							    FileNameLength
-							    =
-							    cifs_strfromUCS_le
-							    (pfindData->
-							     FileName,
-							     (UniChar *)
-							     pfindData->
-							     FileName,
-							     (pfindData->
-							      FileNameLength)
-							     / 2,
-							     cifs_sb->
-							     local_nls);
-						qstring.name =
-						    pfindData->FileName;
-						qstring.len =
-						    pfindData->
-						    FileNameLength;
-						qstring.hash =
-						    full_name_hash(qstring.
-								   name,
-								   qstring.
-								   len);
-						tmp_dentry =
-						    d_lookup(file->
-							     f_dentry,
-							     &qstring);
-						if (tmp_dentry) {
-							existing = TRUE;
-							cFYI(1, ("\nExisting dentry found for "));	/* BB remove */
-							tmp_inode =
-							    tmp_dentry->
-							    d_inode;
-						} else {
-							existing = FALSE;
-							tmp_dentry =
-							    d_alloc(file->
-								    f_dentry,
-								    &qstring);
-							tmp_inode =
-							    new_inode
-							    (file->
-							     f_dentry->
-							     d_sb);
-							tmp_dentry->d_op =
-							    &cifs_dentry_ops;
-							cFYI(1,
-							     ("\nAdding dentry 0x%p with inode 0x%p ",
-							      tmp_dentry,
-							      tmp_inode));
-							d_instantiate
-							    (tmp_dentry,
-							     tmp_inode);
-							if (!existing)
-								d_rehash
-								    (tmp_dentry);
-						}
-						tmp_dentry->d_time =
-						    jiffies;
+        break;
+    default:
+        if (file->private_data == NULL) {      
+                rc = -EBADF;
+                cFYI(1,("\nReaddir on closed srch, pos = %lld",file->f_pos));                
+        } else {	
+                cifsFile = (struct cifsFileInfo *)file->private_data;
+                if(cifsFile->endOfSearch)
+                {
+                        rc = 0;
+                        cFYI(1,("\nEnd of search "));
+                        break;
+                }
+                searchHandle = cifsFile->netfid;                        
+                rc = CIFSFindNext(xid, pTcon, pfindData,&findNextParms, searchHandle, 0,&Unicode, &UnixSearch);
+        		cFYI(1,
+        		     ("Count: %d  End: %d ", findNextParms.SearchCount,
+        		      findNextParms.EndofSearch));       
+                if ((rc == 0) && (findNextParms.SearchCount != 0)) {
+                    for (i = 0; i < findNextParms.SearchCount;i++) {
+                            if (UnixSearch == FALSE) {
+                                if (Unicode == TRUE)
+                                    pfindData->FileNameLength =
+                                        cifs_strfromUCS_le
+                                        (pfindData->FileName,
+                                         (UniChar *)
+                                         pfindData->FileName,
+                                         (pfindData->
+                                          FileNameLength) / 2,
+                                         cifs_sb->local_nls);
+                                qstring.len = pfindData->FileNameLength;
+                                if (((qstring.len != 1)
+                                     || (pfindData->FileName[0] != '.'))
+                                    && ((qstring.len != 2)
+                                    || (pfindData->FileName[0] != '.')
+                                    || (pfindData->FileName[1] != '.'))) {
+                                    cifs_filldir(&qstring, pfindData, file, filldir, direntry);
+                                    file->f_pos++;
+                                }
+                            } else {	/* UnixSearch */
+                                pfindDataUnix =
+                                    (FILE_UNIX_INFO *)pfindData;
+                                if (Unicode == TRUE)
+                                    qstring.len = cifs_strfromUCS_le
+                                        (pfindDataUnix->
+                                         FileName,
+                                         (UniChar *)
+                                         pfindDataUnix->
+                                         FileName,
+                                         MAX_PATHCONF,
+                                         cifs_sb->local_nls);
+                                else
+                                    qstring.len = strnlen(pfindDataUnix->FileName, MAX_PATHCONF);
 
-						tmp_inode->i_blksize =
-						    (pTcon->ses->maxBuf -
-						     MAX_CIFS_HDR_SIZE) &
-						    0xFFFFFE00;
+                                if (((qstring.len != 1)
+                                     || (pfindDataUnix->FileName[0] != '.'))
+                                    && ((qstring.len != 2)
+                                    || (pfindDataUnix->FileName[0] != '.')
+                                    || (pfindDataUnix->FileName[1] != '.'))) {
+                                    cifs_filldir_unix(&qstring, pfindDataUnix, file, filldir, direntry);
+                                    file->f_pos++;
+                                }
+                            }              
+                            pfindData = (FILE_DIRECTORY_INFO *) ((char *) pfindData + pfindData->NextEntryOffset);	/* works also for Unix find struct since this is the first field of both */
+                            /* BB also should check to make sure that pointer is not beyond the end of the SMB */
 
-						fill_in_inode(tmp_inode,
-							      pfindData,
-							      &object_type);
-
-						cFYI(1,
-						     ("\nFile name is %s ",
-						      pfindData->
-						      FileName));
-						if (((pfindData->
-						      FileNameLength != 1)
-						     || (pfindData->
-							 FileName[0] !=
-							 '.'))
-						    &&
-						    ((pfindData->
-						      FileNameLength != 2)
-						     || (pfindData->
-							 FileName[0] !=
-							 '.')
-						     || (pfindData->
-							 FileName[1] !=
-							 '.'))) {
-							rc2 =
-							    filldir
-							    (direntry,
-							     pfindData->
-							     FileName,
-							     pfindData->
-							     FileNameLength,
-							     i +
-							     last_index,
-							     tmp_inode->
-							     i_ino,
-							     object_type);
-							if (rc2)
-								cFYI(1,
-								     (" filldir rc2 = %d ",
-								      rc2));
-							file->f_pos++;
-						}
-
-						pfindData =
-						    (FILE_DIRECTORY_INFO
-						     *) ((char *) pfindData
-							 +
-							 pfindData->
-							 NextEntryOffset);
-						/* BB also check to make sure that it is not beyond the end of the SMB */
-					} else {
-						/* BB fill in Unix search logic info here */
-						if (Unicode == TRUE)
-							qstring.len =
-							    cifs_strfromUCS_le
-							    (pUnixFindData->
-							     FileName,
-							     (UniChar *)
-							     pUnixFindData->
-							     FileName,
-							     MAX_PATHCONF,
-							     cifs_sb->
-							     local_nls);
-						else
-							qstring.len =
-							    strnlen
-							    (pUnixFindData->
-							     FileName,
-							     MAX_PATHCONF);
-
-						qstring.name =
-						    pUnixFindData->
-						    FileName;
-						qstring.hash =
-						    full_name_hash(qstring.
-								   name,
-								   qstring.
-								   len);
-						tmp_dentry =
-						    d_lookup(file->
-							     f_dentry,
-							     &qstring);
-						if (tmp_dentry) {
-							existing = TRUE;
-							cFYI(1, ("\nExisting dentry found for "));	/* BB remove */
-							tmp_inode =
-							    tmp_dentry->
-							    d_inode;
-						} else {
-							existing = FALSE;
-							tmp_dentry =
-							    d_alloc(file->
-								    f_dentry,
-								    &qstring);
-							tmp_inode =
-							    new_inode
-							    (file->
-							     f_dentry->
-							     d_sb);
-							tmp_dentry->d_op =
-							    &cifs_dentry_ops;
-							cFYI(1,
-							     ("\nAdding dentry 0x%p with inode 0x%p ",
-							      tmp_dentry,
-							      tmp_inode));
-							d_instantiate
-							    (tmp_dentry,
-							     tmp_inode);
-							if (!existing)
-								d_rehash
-								    (tmp_dentry);
-						}
-						tmp_dentry->d_time =
-						    jiffies;
-
-						tmp_inode->i_blksize =
-						    (pTcon->ses->maxBuf -
-						     MAX_CIFS_HDR_SIZE) &
-						    0xFFFFFE00;
-
-						unix_fill_in_inode
-						    (tmp_inode,
-						     pUnixFindData,
-						     &object_type);
-
-						cFYI(1,
-						     ("\nFile name is %s ",
-						      pUnixFindData->
-						      FileName));
-						if (((qstring.len != 1)
-						     || (pUnixFindData->
-							 FileName[0] !=
-							 '.'))
-						    && ((qstring.len != 2)
-							|| (pUnixFindData->
-							    FileName[0] !=
-							    '.')
-							|| (pUnixFindData->
-							    FileName[1] !=
-							    '.'))) {
-							rc2 =
-							    filldir
-							    (direntry,
-							     pUnixFindData->
-							     FileName,
-							     qstring.len,
-							     i +
-							     last_index,
-							     tmp_inode->
-							     i_ino,
-							     object_type);
-							if (rc2)
-								cFYI(1,
-								     (" filldir rc2 = %d ",
-								      rc2));
-							file->f_pos++;
-						}
-
-						pUnixFindData =
-						    (FILE_UNIX_INFO
-						     *) ((char *)
-							 pUnixFindData +
-							 pUnixFindData->
-							 NextEntryOffset);
-						/* BB also check to make sure that it is not beyond the end of the SMB */
-					}
-				}	/* end for loop */
-				if (findNextParms.EndofSearch != 0)
-					done_search = TRUE;
-				last_index += i;
-			} else {
-				rc = 0;
-				break;
-			}
-		}		/* end while */
+                    } /* end for loop */
+                    if (findNextParms.EndofSearch != 0) {           
+                            cifsFile->endOfSearch = TRUE;                    
+                    }
+        		} else {
+                    cifsFile->endOfSearch = TRUE;
+        			rc = 0;	/* unless parent directory disappeared - do not return error here (eg Access Denied or no more files) */            
+        		}
+        }
 	}			/* end switch */
-	if (data)
-		kfree(data);
-	if (full_path)
-		kfree(full_path);
-	FreeXid(xid);
-	return rc;
-}
+
+    if (data)
+	    kfree(data);
+    if (full_path)
+	    kfree(full_path);
+    FreeXid(xid);
+
+    return rc;
+}
\ No newline at end of file
