@@ -62,12 +62,23 @@ int cifs_get_inode_info_unix(struct inode **pinode,
 		}
 
 	} else {
+        struct cifsInodeInfo *cifsInfo;
+
 		/* get new inode */
-		if (*pinode == NULL)
+		if (*pinode == NULL) {        
 			*pinode = new_inode(sb);
+            cFYI(1,("\nAlloc new inode %p ",*pinode));
+        }
 		inode = *pinode;
 /*        new_inode = iget(parent_dir_inode->i_sb, findData.IndexNumber); *//* index number not reliable in response data */
 
+        cifsInfo = (struct cifsInodeInfo *) &(inode->u);
+
+        cFYI(1,("\nOld time %ld ",cifsInfo->time));
+        cifsInfo->time = jiffies;
+        cFYI(1,(" New time %ld ",cifsInfo->time));
+        atomic_inc(&cifsInfo->inUse);  /* inc on every refresh of inode */
+
 		inode->i_atime =
 		    cifs_NTtimeToUnix(findData.LastAccessTime);
 		inode->i_mtime =
@@ -162,11 +173,22 @@ int cifs_get_inode_info(struct inode **pinode,
 			return rc;
 		}
 	} else {
-		/* get new inode */
-		if (*pinode == NULL)
+        struct cifsInodeInfo *cifsInfo;
+		
+        /* get new inode */
+		if (*pinode == NULL) {        
 			*pinode = new_inode(sb);
+            cFYI(1,("\nAlloc new inode %p ",*pinode));
+        }
 
 		inode = *pinode;
+        cifsInfo = (struct cifsInodeInfo *) &(inode->u);
+
+        cifsInfo->cifsAttrs = findData.Attributes;
+        cFYI(1,("\nOld time %ld ",cifsInfo->time));
+        cifsInfo->time = jiffies;
+        cFYI(1,(" New time %ld ",cifsInfo->time));
+        atomic_inc(&cifsInfo->inUse);  /* inc on every refresh of inode */
 
 		inode->i_blksize =
 		    (pTcon->ses->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00;
@@ -176,6 +198,7 @@ int cifs_get_inode_info(struct inode **pinode,
 		inode->i_mtime = cifs_NTtimeToUnix(findData.LastWriteTime);
 		inode->i_ctime = cifs_NTtimeToUnix(findData.ChangeTime);
 /* inode->i_mode = S_IRWXUGO;  *//* 777 perms */
+        /* should we treat the dos attribute of read-only as read-only mode bit e.g. 555 */
 		inode->i_mode = S_IALLUGO & ~(S_ISUID | S_IXGRP);	/* 2767 perms indicate mandatory locking - will override for dirs later */
 		cFYI(0,
 		     ("\nAttributes came in as 0x%x\n",
@@ -362,13 +385,28 @@ int cifs_revalidate(struct dentry *direntry)
 	int rc = 0;
 	char *full_path;
 	struct cifs_sb_info *cifs_sb;
+    struct cifsInodeInfo * cifsInode;
 
 	xid = GetXid();
 
 	cifs_sb = (struct cifs_sb_info *) (&(direntry->d_sb->u));
 
 	full_path = build_path_from_dentry(direntry);
-	cFYI(1, (" full path: %s\n", full_path));
+	cFYI(1,
+	     (" full path: %s for inode 0x%p with count %d dentry: 0x%p d_time %ld at time %ld \n", full_path,direntry->d_inode,direntry->d_inode->i_count, direntry,
+	      direntry->d_time, jiffies));
+
+    cifsInode = (struct cifsInodeInfo *) &(direntry->d_inode->u);
+
+
+    if(time_before(jiffies,cifsInode->time + HZ))
+    {
+            cFYI(1,(" Do not need to revalidate "));
+            if (full_path)
+                kfree(full_path);
+            FreeXid(xid);
+            return rc;
+    }
 
 	if (cifs_sb->tcon->ses->capabilities & CAP_UNIX)
 		cifs_get_inode_info_unix(&direntry->d_inode, full_path,
@@ -377,6 +415,8 @@ int cifs_revalidate(struct dentry *direntry)
 		cifs_get_inode_info(&direntry->d_inode, full_path,
 				    direntry->d_sb);
 
+	/* BB if not oplocked, invalidate inode pages if mtime has changed */
+
 	if (full_path)
 		kfree(full_path);
 	FreeXid(xid);
