@@ -788,11 +788,14 @@ int map_smb_to_linux_error(struct smb_hdr *smb)
 		cFYI(1,
 		     (" !!Mapping cifs error code %d ",
 		      smb->Status.CifsError));
-		ntstatus_to_dos(le32_to_cpu(smb->Status.CifsError),
-				&smberrclass, &smberrcode);
+		smb->Status.CifsError = le32_to_cpu(smb->Status.CifsError);
+		ntstatus_to_dos(smb->Status.CifsError, &smberrclass,
+				&smberrcode);
 	} else {
 		smberrclass = smb->Status.DosError.ErrorClass;
-		smberrcode = le16_to_cpu(smb->Status.DosError.Error);
+		smb->Status.DosError.Error =
+		    le16_to_cpu(smb->Status.DosError.Error);
+		smberrcode = smb->Status.DosError.Error;
 	}
 
 	/* old style errors */
