@@ -106,7 +106,8 @@ int smb_send(struct socket *ssocket, struct smb_hdr *smb_buffer,
 	/* smb header is converted in header_assemble. bcc and rest of SMB word area, 
 	   and byte area if necessary, is converted to littleendian in cifssmb.c and RFC1001 
 	   len is converted to bigendian in smb_send */
-
+	if (smb_buf_length > 12)
+		smb_buffer->Flags2 = cpu_to_le16(smb_buffer->Flags2);
 	smb_buffer->smb_buf_length =
 	    cpu_to_be32(smb_buffer->smb_buf_length);
 	cFYI(1, ("\nSending smb of length %d ", smb_buf_length));
@@ -173,8 +174,9 @@ int SendReceive(const unsigned int xid, struct smbSesInfo *ses,
 		cERROR(1, (KERN_ERR "\nCIFS: caught signal"));
 		DeleteMidQEntry(midQ);
 		return -EINTR;
-	} else
+	} else {
 		receive_len = be32_to_cpu(midQ->resp_buf->smb_buf_length);
+	}
 
 	if (timeout == 0) {
 		cFYI(1,
@@ -197,6 +199,17 @@ int SendReceive(const unsigned int xid, struct smbSesInfo *ses,
 
 			out_buf->smb_buf_length =
 			    be32_to_cpu(out_buf->smb_buf_length);
+			if (out_buf->smb_buf_length > 12)
+				out_buf->Flags2 =
+				    le16_to_cpu(out_buf->Flags2);
+			if (out_buf->smb_buf_length > 28)
+				out_buf->Pid = le16_to_cpu(out_buf->Pid);
+			if (out_buf->smb_buf_length > 28)
+				out_buf->PidHigh =
+				    le16_to_cpu(out_buf->PidHigh);
+
+
+			*pbytes_returned = out_buf->smb_buf_length;
 			rc = map_smb_to_linux_error(out_buf);	/* BB watch endianness here BB */
 		} else
 			rc = -EIO;
